//
// $Id: RunAction.cc 171 2008-09-17 14:27:03Z maire $
// 
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "RunAction.hh"
#include "DetectorConstruction.hh"

#include "G4Run.hh"
#include "G4RunManager.hh"
#include "G4UnitsTable.hh"
#include "G4EmCalculator.hh"

#include "Randomize.hh"
#include <iomanip>

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

RunAction::RunAction(DetectorConstruction* det)
:m_detector(det)
{ }

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

RunAction::~RunAction()
{ }

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void RunAction::BeginOfRunAction(const G4Run* run)
{  
  G4cout << "### Run " << run->GetRunID() << " start." << G4endl;
  
  //initialize total energy deposit
  //
  m_edepTotal = 0.; 
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void RunAction::EndOfRunAction(const G4Run*)
{ 
  // choose printing format
  std::ios::fmtflags mode = G4cout.flags();
  G4cout.setf(std::ios::fixed,std::ios::floatfield);
  G4int prec = G4cout.precision(3);
  
  //print total energy deposit
  //
  G4cout 
    << "\n ---> Total Energy deposited = " << G4BestUnit(m_edepTotal,"Energy")
    << G4endl;
    
  //get mass of astronaut
  //
  G4double mass = m_detector->GetAstronaut()->GetLogicalVolume()->GetMass();
  G4cout << " ---> Mass = " << G4BestUnit(mass,"Mass") << G4endl;
  
  //compute dose
  //
  G4double dose = m_edepTotal/mass;
  G4cout.setf(mode,std::ios::floatfield);
  G4cout.precision(5);     
  G4cout << " ---> Dose = " << G4BestUnit(dose,"Dose") 
         << " (" << dose/(MeV/kg) << " MeV/kg)" << G4endl;
  
      
  // restaure default formats
  G4cout.setf(mode,std::ios::floatfield);
  G4cout.precision(prec);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
