//
// $Id: SteppingAction.cc 169 2008-09-16 16:16:21Z maire $
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "SteppingAction.hh"
#include "DetectorConstruction.hh"
#include "RunAction.hh"
#include "EventAction.hh"

#include "G4Step.hh"
#include "G4Navigator.hh"
#include "Randomize.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

SteppingAction::SteppingAction(DetectorConstruction* det, RunAction* run,
                                                          EventAction* event)
:m_detector(det),m_runAction(run),m_eventAction(event)
{ }

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

SteppingAction::~SteppingAction()
{ }

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void SteppingAction::UserSteppingAction(const G4Step* step)
{
 // in which volume is the step ?
 G4TouchableHandle touchable = step->GetPreStepPoint()->GetTouchableHandle();
 G4VPhysicalVolume* volume   = touchable->GetVolume();

 // energy deposit in this step 
 G4double edep = step->GetTotalEnergyDeposit();

 // is astronaut ?
 if (volume == m_detector->GetAstronaut()) {
 
   // sum edep in EventAction
   m_eventAction->AddEdep(edep);
   
   // get step points in world coordinate system
   G4ThreeVector point1 = step->GetPreStepPoint()->GetPosition();
   G4ThreeVector point2 = step->GetPostStepPoint()->GetPosition();
   
   // randomize point of energy deposition
   G4ThreeVector pointE = point1 + G4UniformRand()*(point2 - point1);      
   // transform it in local coordinate system
   G4ThreeVector localPointE 
   = touchable->GetHistory()->GetTopTransform().TransformPoint(pointE);
   
   // extract z and offset
   G4double zLocal = 0.5*(m_detector->GetAstronautHeight()) - localPointE.z();
   
   // sum edep in RunAction
   m_runAction->SumDepthDose(zLocal,edep);
 }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
