//
// $Id: DetectorMessenger.cc 168 2008-09-16 13:23:52Z maire $
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "DetectorMessenger.hh"

#include "DetectorConstruction.hh"
#include "G4UIdirectory.hh"
#include "G4UIcmdWithAString.hh"
#include "G4UIcmdWithADoubleAndUnit.hh"
#include "G4UIcmdWithoutParameter.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

DetectorMessenger::DetectorMessenger(DetectorConstruction * Det)
:m_detector(Det)
{ 
  m_detDir = new G4UIdirectory("/tutorial/det/");
  m_detDir->SetGuidance("detector construction commands");

  m_materialCmd = new G4UIcmdWithAString ("/tutorial/det/spacecraftMaterial", this);
  m_materialCmd->SetGuidance("Select spacecraft material");
  m_materialCmd->SetParameterName("spacecraftMaterial",false);

  m_thicknessCmd = new G4UIcmdWithADoubleAndUnit("/tutorial/det/spacecraftThickness",this);
  m_thicknessCmd->SetGuidance("Select spacecraft thickness");
  m_thicknessCmd->SetParameterName("spacecraftThickness",false);
  m_thicknessCmd->SetRange("spacecraftThickness>0.");
  m_thicknessCmd->SetUnitCategory("Length");
  
  m_radiusCmd = new G4UIcmdWithADoubleAndUnit ("/tutorial/det/cabinRadius", this);
  m_radiusCmd->SetGuidance("Select spacecraft radius");
  m_radiusCmd->SetParameterName("cabinRadius",false);
  m_radiusCmd->SetUnitCategory("Length");

  m_lengthCmd = new G4UIcmdWithADoubleAndUnit ("/tutorial/det/cabinLength", this);
  m_lengthCmd->SetGuidance("Select spacecraft length");
  m_lengthCmd->SetParameterName("spacecraftLength",false);
  m_lengthCmd->SetRange("cabinLength>0.&&cabinLength<20*m");
  m_lengthCmd->SetUnitCategory("Length");
     
  m_updateCmd = new G4UIcmdWithoutParameter("/tutorial/det/update",this);
  m_updateCmd->SetGuidance("force to recompute geometry.");
  m_updateCmd->SetGuidance("This command MUST be applied before \"beamOn\" ");
  m_updateCmd->SetGuidance("if you changed geometrical value(s).");
  m_updateCmd->AvailableForStates(G4State_Idle);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

DetectorMessenger::~DetectorMessenger()
{
  delete m_materialCmd;
  delete m_thicknessCmd;  
  delete m_radiusCmd;
  delete m_lengthCmd;
  delete m_updateCmd;
  
  delete m_detDir;  
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void DetectorMessenger::SetNewValue(G4UIcommand* command,G4String newValue)
{
  if( command == m_materialCmd )
   { m_detector->SetSpacecraftMaterial(newValue);}
     
  if( command == m_thicknessCmd )
   { m_detector->SetSpacecraftThickness(m_thicknessCmd->GetNewDoubleValue(newValue));} 
            
  if( command == m_radiusCmd )
   { m_detector->SetCabinRadius(m_radiusCmd->GetNewDoubleValue(newValue));}
   
  if( command == m_lengthCmd )
   { m_detector->SetCabinLength(m_lengthCmd->GetNewDoubleValue(newValue));}
              
  if( command == m_updateCmd )
   { m_detector->UpdateGeometry();}
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
