//
// $Id: RunAction.hh 166 2008-09-16 10:39:56Z maire $
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#ifndef RunAction_h
#define RunAction_h 1

#include "G4UserRunAction.hh"
#include "globals.hh"

#include <vector>

class G4Run;
class DetectorConstruction;

namespace AIDA {
  class IAnalysisFactory;
  class ITree;
  class IHistogram1D;
}
  
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

class RunAction : public G4UserRunAction
{
  public:
    RunAction(DetectorConstruction*);
   ~RunAction();

  public:
    void BeginOfRunAction(const G4Run*);
    void   EndOfRunAction(const G4Run*);
    
    void SumEvents(G4double e) { m_sumEdep += e; m_sumEdep2 += e*e; m_nbEntry++; };
    void SumDepthDose(G4double,G4double);
    
    void WriteDepthDose(G4String);
    
    void BookHistoFile (G4String, G4String);
    void SaveHistoFile ();    
    AIDA::IHistogram1D* GetHisto(G4int id) {return m_histo[id];}
    		               
  private:
    DetectorConstruction* m_detector;
    
    G4double m_sumEdep, m_sumEdep2;
    G4int m_nbEntry;
    
    std::vector<G4double> m_depthDose;
    G4double m_astronautHeight;
    G4double m_sliceWidth;
    G4int    m_nbSlices;
 
    G4String histoFile;   
    AIDA::IAnalysisFactory* m_af;         
    AIDA::ITree*            m_tree;             
    AIDA::IHistogram1D*     m_histo[2];            
};

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#endif

