#include "SensitiveDetector.hh"
#include "DetectorConstruction.hh"
#include "RunAction.hh"

#include "G4Step.hh"
#include "Randomize.hh"
#include "G4RunManager.hh"

SensitiveDetector::SensitiveDetector(G4String SDname)
  : G4VSensitiveDetector(SDname)
{
  // -- obtain detector construction and run action pointers through the run manager
  m_detector = (DetectorConstruction*)(G4RunManager::GetRunManager()->GetUserDetectorConstruction());
  m_runAction = (RunAction*)(G4RunManager::GetRunManager()->GetUserRunAction());
}

SensitiveDetector::~SensitiveDetector()
{}

G4bool SensitiveDetector::ProcessHits(G4Step *step, G4TouchableHistory *)
{
  // -- step in garanteed to be is astronaut's volume : no need to check for volume
  
  G4TouchableHandle touchable = step->GetPreStepPoint()->GetTouchableHandle();
  // energy deposit in this step 
  G4double edep = step->GetTotalEnergyDeposit();

  if (edep <= 0.) return false;

  // get step points in world coordinate system
  G4ThreeVector point1 = step->GetPreStepPoint()->GetPosition();
  G4ThreeVector point2 = step->GetPostStepPoint()->GetPosition();
  
  // randomize point of energy deposition
  G4ThreeVector pointE = point1 + G4UniformRand()*(point2 - point1);      
  // transform it in local coordinate system
  G4ThreeVector localPointE 
    = touchable->GetHistory()->GetTopTransform().TransformPoint(pointE);
  
  // extract z and offset
  G4double zLocal = 0.5*(m_detector->GetAstronautHeight()) - localPointE.z();
  
  // sum edep in RunAction (in SD vector):
  m_runAction->SumDepthDoseSD(zLocal,edep);

  return true;


  // ********************************************************************
  //                      SteppingAction code
  /* --->>

  // in which volume is the step ?
  G4TouchableHandle touchable = step->GetPreStepPoint()->GetTouchableHandle();
  G4VPhysicalVolume* volume   = touchable->GetVolume();
  
  // energy deposit in this step 
  G4double edep = step->GetTotalEnergyDeposit();
  
  // is astronaut ?
  if (volume == m_detector->GetAstronaut()) {
    ////G4cout << "--> edep in astronaut : " << edep/keV << " keV" << G4endl;
    
    // sum edep in EventAction
    m_eventAction->AddEdep(edep);
    
    //---------for depth dose distribution---------------------------------------
    
    // get step points in world coordinate system
    G4ThreeVector point1 = step->GetPreStepPoint()->GetPosition();
    G4ThreeVector point2 = step->GetPostStepPoint()->GetPosition();
    
    // randomize point of energy deposition
    G4ThreeVector pointE = point1 + G4UniformRand()*(point2 - point1);      
    // transform it in local coordinate system
    G4ThreeVector localPointE 
      = touchable->GetHistory()->GetTopTransform().TransformPoint(pointE);
    
    // extract z and offset
    G4double zLocal = 0.5*(m_detector->GetAstronautHeight()) - localPointE.z();
    
    // sum edep in RunAction
    m_runAction->SumDepthDose(zLocal,edep);
    
    //----------end of depth dose distribution-------------------------------------
  }

  <<--- */

}
