//
// $Id: EventAction.cc 170 2008-09-17 09:20:45Z maire $
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "EventAction.hh"

#include "RunAction.hh"

#include "G4Event.hh"
#include "G4UnitsTable.hh"

#include "G4SDManager.hh"
#include "G4RunManager.hh"
#include "AstroHitCollection.hh"
#include "DetectorConstruction.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

EventAction::EventAction(RunAction* run)
:m_runAction(run), m_printModulo(5000)
{}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

EventAction::~EventAction()
{}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void EventAction::BeginOfEventAction(const G4Event* evt)
{
 G4int evtNb = evt->GetEventID();

 //printing survey
 if (evtNb%m_printModulo == 0)
    G4cout << "\n---> Begin of Event: " << evtNb << G4endl;
    
 //energy deposited per event
 m_edepPerEvent = 0.;   
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void EventAction::EndOfEventAction(const G4Event *event)
{
  //energy deposited per event
  //
  if (m_edepPerEvent > 0.) {
   m_runAction->SumEvents(m_edepPerEvent);
  }

  // -- jour4b2:
  // -------------------------------
  // -- Get back collection pointer:
  // -------------------------------
  // -- find back collection, this is made by getting the collection ID:
  G4SDManager*             SDmanager = G4SDManager::GetSDMpointer();
  G4int        astronautCollectionID = SDmanager->GetCollectionID("AstronautCollection");
  AstroHitCollection*           astroCollection(0);
  // -- Collection ID is correct, we get the pointer of the collection:
  if (astronautCollectionID>=0) astroCollection = (AstroHitCollection*)(event->GetHCofThisEvent()->GetHC(astronautCollectionID));
  else G4cout << "Collection `AstronautCollection' not found !!!!" << G4endl;

  // -- Question : why do we need to cast (AstroHitCollection*) two lines above ?

  // --------------------------------------------
  // -- print hits found for the 5 first events:
  // --------------------------------------------
  if (event->GetEventID() < 5)
    {
      G4cout << "Event::EndOfEventAction(...) for event # " << event->GetEventID() << " : " << G4endl;
      // -- get back detector pointer to get astronaut mass:
      DetectorConstruction* detector = (DetectorConstruction*)(G4RunManager::GetRunManager()->GetUserDetectorConstruction());
      G4double mass = detector->GetAstronaut()->GetLogicalVolume()->GetMass();
      if (astroCollection)
	{
	  // -- Now print:
	  G4cout << " - number of hits : " << astroCollection->entries() << G4endl;
	  if  (astroCollection->entries()) G4cout << " - hit list : " << G4endl;
	  else                             G4cout << " - hit list : (empty)" << G4endl;
	  for (G4int i=0; i<astroCollection->entries(); i++)
	    {
	      AstroHit* hit = (*astroCollection)[i];
	      G4cout << "   k = " << hit->GetSlice() 
		     << " , dose (gray) = "  << hit->GetEnergy()/mass/gray 
		     << " , nb protons : "   << hit->GetNbProton()
		     << " , <eKin>(MeV) = "  << hit->GetProtonMeanEkin()/MeV << G4endl;
	    }
	}
    }

  // ------------------------------------------
  // -- fill the RunAction table for SD vector:
  // ------------------------------------------
  if (astroCollection)
    {
      for (G4int i=0; i<astroCollection->entries(); i++)
	{
	  AstroHit* hit = (*astroCollection)[i];
	  m_runAction->SumDepthDoseSD(hit->GetSlice(),
				      hit->GetEnergy(),
				      hit->GetNbProton(),
				      hit->GetProtonTotalEkin());
	}
    }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......


