#ifndef AstroHit_h
#define AstroHit_h 1

// ---------------------------------------------------------------------------
// -- We define "our" hit format : it is caracterized by an G4int value, which
// -- is the slice number, and an energy value, the accumulated energy in this
// -- slice.
// ---------------------------------------------------------------------------

#include "G4VHit.hh"
#include "G4Allocator.hh"

class AstroHit : public G4VHit {
public:
  AstroHit(G4int slice);
  ~AstroHit();
  
public:
  // -- The new and delete operators are overloaded for performances reasons:
  // -- Tricky business here... :-(, but provided for you below
  inline void *operator    new(size_t);
  inline void  operator delete(void *aHit);

public:
  void                       addE(double e)            {m_e += e;}
  void           accumulateProton(G4double eKin)       {m_eKproton += eKin, m_nbProton++;}
  G4double              GetEnergy()              const {return m_e                    ;}
  G4double     GetProtonTotalEkin()              const {return m_eKproton             ;}
  G4double      GetProtonMeanEkin()              const {G4double mean(-1.); if (m_nbProton>0) mean = m_eKproton / m_nbProton; return mean;}
  G4int               GetNbProton()              const {return m_nbProton             ;}
  G4int                  GetSlice()              const {return m_slice;}

private:
  const G4int m_slice;
  G4double    m_e;
  G4double    m_eKproton;
  G4int       m_nbProton;
};


// -- new and delete overloaded operators:
extern G4Allocator<AstroHit> AstroHitAllocator;

inline void* AstroHit::operator new(size_t)
{
  void *aHit;
  aHit = (void *) AstroHitAllocator.MallocSingle();
  return aHit;
}
inline void AstroHit::operator delete(void *aHit)
{
  AstroHitAllocator.FreeSingle((AstroHit*) aHit);
}

#endif
