//
// $Id: PrimaryGeneratorAction.cc 222 2008-11-21 08:12:11Z jacquem $
//
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "PrimaryGeneratorAction.hh"

#include "DetectorConstruction.hh"
#include "PrimaryGeneratorMessenger.hh"

#include "G4Event.hh"
#include "G4ParticleGun.hh"
#include "G4ParticleTable.hh"
#include "G4ParticleDefinition.hh"

#include "Randomize.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

PrimaryGeneratorAction::PrimaryGeneratorAction(DetectorConstruction* det)
:m_detector(det)					       
{
  m_particleGun  = new G4ParticleGun(1);
  G4ParticleDefinition* particle
           = G4ParticleTable::GetParticleTable()->FindParticle("proton");
  m_particleGun->SetParticleDefinition(particle);
  m_particleGun->SetParticleEnergy(500*MeV);
  
  m_rndmFlag = "on";
  
  //create a messenger for this class
  m_gunMessenger = new PrimaryGeneratorMessenger(this);    
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

PrimaryGeneratorAction::~PrimaryGeneratorAction()
{
  delete m_particleGun;
  delete m_gunMessenger;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void PrimaryGeneratorAction::GeneratePrimaries(G4Event* anEvent)
{
  //this function is called at the begining of event
  //
  G4double y0 = (m_detector->GetWorldSizeXY())*0.5;
  G4double x0 = 0., z0 = 0.;
  if (m_rndmFlag == "on") {
    x0 = (m_detector->GetWorldSizeXY())*(G4UniformRand()-0.5);
    z0 = (m_detector->GetWorldSizeZ()) *(G4UniformRand()-0.5);
  }
     
  m_particleGun->SetParticlePosition(G4ThreeVector(x0,y0,z0));     
  m_particleGun->SetParticleMomentumDirection(G4ThreeVector(0.,-1.,0.));  
  m_particleGun->GeneratePrimaryVertex(anEvent);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

